# -*- coding: utf-8 -*-
"""

"""

import time
import uGao.MPlot as mp
import uGao.M2PlotPlusCui as mp2
import Manyo

# 予約語の定義
ret = None 
vp1 = None
vp2 = None
vc = None
vc2 = None
vc3 = None
ec = None
earray = None
DAT = None
ECM = None
ECA = None
vcm = None
        
###################################################                            
def DetectMap(matrix=DAT):
        """
        Display Detector Map
        @param  matrix [Def:DAT] ElementContainerMatrix Object 
        @retval vp3
        """
        vp3=mp2.M2PlotPlusCui(matrix)
        vp3.ShowData()
        num_x=matrix.PutSize()
        num_y=matrix(0).PutSize()
        width = int( 1000 )
        height = int( float(width) * float(num_y)/float(num_x) )
        vp3.SetWindowSize( width, height )
        return vp3

###################################################
def DetectMapChangeData(matrix=DAT):
        """
        Display Detector Map
        @param  matrix [Def:DAT] ElementContainerMatrix Object
        @retval vp3
        """
        vp3.ChangeData(matrix)
        

###################################################                         
def MPlot(ec=ec):
        """
        Create 1D Plotter and Draw EC
        @param  ec [Def:EC]  ElementContainer
        @param  psd      PSD No.    
        @param  pixel    Pixel No.    
        @retval vp1    Instance of Plotter
        """ 
        # 親プロセス終了時に、プロッタを終了      
        return mp.MPlot(ec, True)
    
###################################################                            
def AddToMPlot(vp1=vp1, ec=ec):
        """
        Add EC to 1D Plotter
        @param  vp1 [Def:vp1]     Instance of 1D Plotter
        @param  ec [Def:EC]      ElementContainer
        @retval None
        """ 
        vp1.AddData(ec)  
        
###################################################                         
def M2PlotPlus(matrix=ECA):
        """
        Create 2D Plotter and display 2D Map
        @param  matrix [Def:ECA]  ElementContainerArray
        @retval vp2 [Def:vp2]  Instance of 2D Plotter
        """ 
        # 親プロセス終了時に、プロッタを終了      
        return mp2.M2PlotPlusCui(matrix)

###################################################
def ChangeDataM2Plot(vp2=vp2, matrix=ECA):
        """
        Change 2D Map of M2Plot
	@param  vp2 [Def:vp2]     Instance of 2D Plotter
        @param  matrix [Def:ECA]  ElementContainerArray
        @retval None
        """ 
        # Map データの変更
        if isinstance(matrix,Manyo.ElementContainerMatrix):
                vp2.ChangeData(matrix.Put(0))
        elif isinstance(matrix,Manyo.ElementContainerArray):
                vp2.ChangeData(matrix)
        else:
                return


           

# 関数名登録ディクショナリ
_functions = {"DetectMap":DetectMap,
              "DetectMapChangeData":DetectMapChangeData,
              "MPlot":MPlot,
              "AddToMPlot":AddToMPlot,
              "M2PlotPlus":M2PlotPlus,
	      "ChangeDataM2Plot":ChangeDataM2Plot
	      }

_functionsOrder = [
	"DetectMap",
        "DetectMapChangeData",
	"MPlot",
	"AddToMPlot",
	"M2PlotPlus",
	"ChangeDataM2Plot"
	]
         



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function

#from fw.common.Result import * ##[inamura 140204]
import os, datetime
from xml.dom.minidom import parse
import Manyo.Utsusemi as mu

# 原子情報ファイル名
DEF_ATOM_DB_FILE_NAME = 'atomDBTaikan.xml'

# 辞書のキー名
DEF_ATOM_DB_KEY_DICTIONARY = 'dictionary'

# タグ名
DEF_ATOM_DB_ATOMDBTAIKAN_TAG = 'AtomDBTaikan'					# AtomDBTaikan
DEF_ATOM_DB_NO_TAG = 'No'										# No
DEF_ATOM_DB_INCB_TAG = 'Incb'									# Incb
DEF_ATOM_DB_COHB_TAG = 'Cohb'									# Cohb
DEF_ATOM_DB_COHXS_TAG = 'Cohxs'									# Cohxs
DEF_ATOM_DB_INCXS_TAG = 'Incxs'									# Incxs
DEF_ATOM_DB_SCATTXS_TAG = 'Scattxs'								# Scattxs
DEF_ATOM_DB_ABSXS_TAG = 'Absxs'									# Absxs
DEF_ATOM_DB_ISOTOPE_TAG = 'Isotope'								# Isotope

# ViewRed, ViewGreen, ViewBlueを全て取得したいときに指定するタグ名
DEF_ATOM_DB_VIEWCOLORS_TAG = 'Viewcolors'

# 属性名
DEF_ATOM_DB_ID_TAG = 'Id'										# Id
DEF_ATOM_DB_ATOMNAME_TAG = 'AtomName'							# AtomName
DEF_ATOM_DB_MOLWEIGHT_TAG = 'MolWeight'							# MolWeight
DEF_ATOM_DB_VIEWRED_TAG = 'ViewRed'								# ViewRed
DEF_ATOM_DB_VIEWGREEN_TAG = 'ViewGreen'							# ViewGreen
DEF_ATOM_DB_VIEWBLUE_TAG = 'ViewBlue'							# ViewBlue
DEF_ATOM_DB_VIEWSIZE_TAG = 'ViewSize'							# ViewSize
DEF_ATOM_DB_NAME_TAG = 'Name'									# Name
DEF_ATOM_DB_MASS_TAG = 'Mass'									# Mass
DEF_ATOM_DB_CONC_TAG = 'Conc'									# Conc
DEF_ATOM_DB_HALFLIFE_TAG = 'HalfLife'							# HalfLife
DEF_ATOM_DB_UNC_TAG = 'Unc'										# Unc

# 戻り値
DEF_ATOM_DB_GET_VALUE = 0										# 値取得
DEF_ATOM_DB_GET_PARENT_VALUE = 1								# 親の値を返す
DEF_ATOM_DB_GET_ISOTOPE_VALUE = 2								# 同位体の値を返す
DEF_ATOM_DB_ARG_ERROR = 3										# 引数間違い
DEF_ATOM_DB_NOVAL = 4											# 値なし

##[inamura 140204]-->
RSLT_OK = 'OK'
RSLT_NG = 'NG'
class Result(object):
	def __init__(self, status):
		self.status = status
##<--[inamura 140204]

###############################################################################
# 定数値定義
###############################################################################


###############################################################################
# 大観用原子情報DBクラス
###############################################################################
class AtomDBTaikanClass(object):
	def __init__(self):
		"""
		コンストラクタ

		@param	self					自身のインスタンス
		@retVal	なし
		"""
		# AtomDBファイルから取得する原子情報を入れるテーブルの初期化
		self.atomIDNameList = []
		self.atomInfo = []
		self.isoTopeInfo = []

		# 共有データを特定する為のキー
		self.__key = ''

	###########################################################################
	def ReadDBFile(self, filepath = ""):
		"""
		原子情報DBデータ取得処理
		AtomDBTaikanからパラメータを取得し属性へ設定する。
   		@param	self									自身のインスタンス
		@param	params				(Element)			パラメータ
		"""
	#	## XML file読み込み
	#	if "" == filepath:
	#		# ファイルパスの指定がない場合は固定のパスを使用する。
	#		xmlpath = os.path.join(os.environ["MLF_USR_DIR"], "ana/xml", DEF_ATOM_DB_FILE_NAME)
	#	else:
	#		xmlpath = filepath

		## XML ファイルパス生成
		xmlpath = self.__MakeDBFilePath(filepath)

		## XML file読み込み
		if not os.path.exists(xmlpath):
			print("AtomDBTaikan.xml is not exist. Pass = ", xmlpath)
	#		raise Exception
			return Result(RSLT_NG)
		else:
			xmlfile = parse(xmlpath)

		# 共有データを特定する為のキーとして、AtomDBTaikan.xmlのフルパスを使用する
		self.__key = self.MakeKey(xmlpath)

		# AtomDBTaikanタグの取得
		atomDBTaikanInfo = xmlfile.getElementsByTagName(DEF_ATOM_DB_ATOMDBTAIKAN_TAG)[0]

		# No タグの取得
		noInfoList = atomDBTaikanInfo.getElementsByTagName(DEF_ATOM_DB_NO_TAG)

		for i, noInfoXml in enumerate(noInfoList):

			# 原子の辞書を作成し、リストに追加。
			atom_dic = self.MakeAtomDictionary(noInfoXml)
			if {} != atom_dic:
				self.atomInfo.append(atom_dic)

			# Isotope タグの取得
			isotopeInfoList = noInfoXml.getElementsByTagName(DEF_ATOM_DB_ISOTOPE_TAG)
			isotope_dic_list = []
			for j, isotopeInfoXml in enumerate(isotopeInfoList):
				# 同位体の辞書を作成し、リストに追加。
				isotope_dic_list.append(self.MakeIsotopeDictionary(isotopeInfoXml))

			# idとAtomNameが存在したら、同位体リストの辞書を作成し、リストに追加する。
			if DEF_ATOM_DB_ID_TAG in atom_dic and DEF_ATOM_DB_ATOMNAME_TAG in atom_dic:
				isotope_dic = {}
				isotope_dic[DEF_ATOM_DB_ID_TAG] = atom_dic[DEF_ATOM_DB_ID_TAG]
				isotope_dic[DEF_ATOM_DB_ATOMNAME_TAG] = atom_dic[DEF_ATOM_DB_ATOMNAME_TAG]
				isotope_dic[DEF_ATOM_DB_KEY_DICTIONARY] = isotope_dic_list
				self.isoTopeInfo.append(isotope_dic)

			#print 'self.atomIDNameList = ', self.atomIDNameList
			#print 'self.atomInfo = ', self.atomInfo
			#print 'self.isoTopeInfo = ', self.isoTopeInfo

	###########################################################################
	def MakeAtomDictionary(self, noInfoXml):
		"""
		Noのタグ下のXML情報から原子情報の辞書を作成する
		@param	self									自身のインスタンス
		@param	noInfoXml		(XML)					No下のXML情報
		@retval					(dictionary)			原子情報の辞書
		"""
		atr_dic = {}
		info_dic = {}
		id_name_list = []
		atom_dic = {}
		# No下の属性情報を取得する。
		# [id,AtomName,MolWeight,ViewRed,ViewGreen,ViewBlue,ViewSize]のリストになっている。
		atr_dic = self.GetNoAttribute(noInfoXml)

		# No下の原子情報を取得する。
		# [Cohb,Incb,Cohxs,Incxs,Scattxs,Absxs]のリストになっている。
		info_dic = self.GetAtomInfo(noInfoXml)
		# idとAtomNameを取得し、リスト化する。
		if DEF_ATOM_DB_ID_TAG in atr_dic and DEF_ATOM_DB_ATOMNAME_TAG in atr_dic:
			id_name_list.append(atr_dic[DEF_ATOM_DB_ID_TAG])
			id_name_list.append(atr_dic[DEF_ATOM_DB_ATOMNAME_TAG])
			self.atomIDNameList.append(id_name_list)

			# 原子の辞書を作成。
			atom_dic.update(atr_dic)
			atom_dic.update(info_dic)

		#print 'id_name_list = ', id_name_list
		#print 'atom_dic = ', atom_dic

		return atom_dic

	###########################################################################
	def MakeIsotopeDictionary(self, noInfoXml):
		"""
		Isotopeのタグ下のXML情報から同位体情報の辞書を作成する
		@param	self									自身のインスタンス
		@param	noInfoXml		(XML)					Isotope下のXML情報
		@retval					(dictionary)			同位体情報の辞書
		"""
		atr_dic = {}
		info_dic = {}
		isotope_dic = {}
		# Isotope下の属性情報を取得する。
		# [Name,Mass,Conc,ViewRed,ViewGreen,ViewBlue,ViewSize]の辞書になっている。
		atr_dic = self.GetIsotopeAttribute(noInfoXml)

		# Isotope下の原子情報を取得する。
		# [Cohb,Incb,Cohxs,Incxs,Scattxs,Absxs]の辞書になっている。
		info_dic = self.GetAtomInfo(noInfoXml)

		# 同位体の辞書を作成。
		isotope_dic.update(atr_dic)
		isotope_dic.update(info_dic)

		#print 'isotope_dic = ', isotope_dic

		return isotope_dic

	###########################################################################
	def MakeKey(self, filepath):
		"""
		データ情報の値に応じてユニークなキーを作成する

		キーのフォーマットは任意
		"""
		return self.__MakeDBFilePath(filepath).replace( '\\', '/' )

	###########################################################################
	def GetKey(self):
		"""
		ユニークなキーを取得する
		※キーのフォーマットは任意

		@param self クラスのインスタンス
		@retval     キー
		"""
		return self.__key

	###########################################################################
	def __MakeDBFilePath(self, filepath):
		"""
		AtomDBTaikan.xmlのフルパス生成

		@param self      クラスのインスタンス
		@param filepath  AtomDBTaikan.xmlのフルパス
		@retval          AtomDBTaikan.xmlのフルパス
		"""
		if "" == filepath:
			# ファイルパスの指定がない場合は固定のパスを使用する。
			#xmlpath = os.path.join(os.environ["UTSUSEMI_USR_DIR"], "ana","xml", DEF_ATOM_DB_FILE_NAME) ##[inamura 140204]
                        xmlpath = mu.FindParamFilePath( DEF_ATOM_DB_FILE_NAME )
                        if xmlpath=="":
                                raise UserWarning("DBfile not found."+DEF_ATOM_DB_FILE_NAME)
		else:
			xmlpath = filepath

		return xmlpath

	###########################################################################
	def GetNoAttribute(self, no_info_xml):
		"""
		Noの属性として持っている情報をXMLから取得する
		「id」、「AtomName」、「MolWeight」、「ViewRed」、「ViewGreen」、「ViewBlue」、
		「ViewSize」を取得し、辞書化して戻り値として返す。
		@param	self									自身のインスタンス
		@param	noinfoxml		(XML)					No下のXML情報
		@retval					(dictionary)			{id,AtomName,MolWeight,
											ViewRed,ViewGreen,ViewBlue,ViewSize}
		"""
		info_dic = {}
		# 属性から取得。
		no_id = no_info_xml.getAttribute(DEF_ATOM_DB_ID_TAG)
		atom_name = no_info_xml.getAttribute(DEF_ATOM_DB_ATOMNAME_TAG)
		molweight = no_info_xml.getAttribute(DEF_ATOM_DB_MOLWEIGHT_TAG)
		view_red = no_info_xml.getAttribute(DEF_ATOM_DB_VIEWRED_TAG)
		view_green = no_info_xml.getAttribute(DEF_ATOM_DB_VIEWGREEN_TAG)
		view_blue = no_info_xml.getAttribute(DEF_ATOM_DB_VIEWBLUE_TAG)
		view_size = no_info_xml.getAttribute(DEF_ATOM_DB_VIEWSIZE_TAG)

		# リストに追加。
		info_dic[DEF_ATOM_DB_ID_TAG] = no_id
		info_dic[DEF_ATOM_DB_ATOMNAME_TAG] = atom_name
		info_dic[DEF_ATOM_DB_MOLWEIGHT_TAG] = molweight
		info_dic[DEF_ATOM_DB_VIEWRED_TAG] = view_red
		info_dic[DEF_ATOM_DB_VIEWGREEN_TAG] = view_green
		info_dic[DEF_ATOM_DB_VIEWBLUE_TAG] = view_blue
		info_dic[DEF_ATOM_DB_VIEWSIZE_TAG] = view_size

		return info_dic

	###########################################################################
	def GetIsotopeAttribute(self, no_info_xml):
		"""
		Isotopeの属性として持っている情報をXMLから取得する
		「Name」、「Mass」、「Conc」、「ViewRed」、「ViewGreen」、「ViewBlue」、
		「ViewSize」を取得し、辞書化して戻り値として返す。
		@param	self									自身のインスタンス
		@param	noinfoxml		(XML)					No下のXML情報
		@retval					(dictionary)			{Name,Mass,Conc,ViewRed,
													ViewGreen,ViewBlue,ViewSize}
		"""
		info_dic = {}
		# 属性から取得。
		name = no_info_xml.getAttribute(DEF_ATOM_DB_NAME_TAG)
		mass = no_info_xml.getAttribute(DEF_ATOM_DB_MASS_TAG)
		conc = no_info_xml.getAttribute(DEF_ATOM_DB_CONC_TAG)
		halflife = no_info_xml.getAttribute(DEF_ATOM_DB_HALFLIFE_TAG)
		view_red = no_info_xml.getAttribute(DEF_ATOM_DB_VIEWRED_TAG)
		view_green = no_info_xml.getAttribute(DEF_ATOM_DB_VIEWGREEN_TAG)
		view_blue = no_info_xml.getAttribute(DEF_ATOM_DB_VIEWBLUE_TAG)
		view_size = no_info_xml.getAttribute(DEF_ATOM_DB_VIEWSIZE_TAG)

		# リストに追加。
		info_dic[DEF_ATOM_DB_NAME_TAG] = name
		info_dic[DEF_ATOM_DB_MASS_TAG] = mass
		info_dic[DEF_ATOM_DB_CONC_TAG] = conc
		info_dic[DEF_ATOM_DB_HALFLIFE_TAG] = halflife
		info_dic[DEF_ATOM_DB_VIEWRED_TAG] = view_red
		info_dic[DEF_ATOM_DB_VIEWGREEN_TAG] = view_green
		info_dic[DEF_ATOM_DB_VIEWBLUE_TAG] = view_blue
		info_dic[DEF_ATOM_DB_VIEWSIZE_TAG] = view_size

		return info_dic

	###########################################################################
	def GetAtomInfo(self, no_info_xml):
		"""
		原子情報をXMLから取得する
		「Cohb」、「Incb」、「Cohxs」、「Incxs」、「Scattxs」、「Absxs」を
		取得し、辞書化して戻り値として返す。
		@param	self									自身のインスタンス
		@param	noinfoxml		(XML)					No下のXML情報
		@retval					(dictionary)			{Cohb,Incb,Cohxs,Incxs,
																Scattxs,Absxs}
		"""
		info_dic = {}

		# No下の直下のElementの子ノードを取得する。
		choices = [e for e in no_info_xml.childNodes if e.nodeType == e.ELEMENT_NODE]

		# No下のCohb、Incb、Cohxs、Incxs、Scattxs、Absxsをタグ指定で取得。
		for i, choice in enumerate(choices):
			#print 'i =', i
			#print 'choice =', choice
			tagname = choice.tagName
			val = choice.firstChild.data
			atr_val = choice.getAttribute(DEF_ATOM_DB_UNC_TAG)

			if DEF_ATOM_DB_COHB_TAG == tagname or DEF_ATOM_DB_INCB_TAG == tagname \
				or DEF_ATOM_DB_COHXS_TAG == tagname or DEF_ATOM_DB_INCXS_TAG == tagname \
				or DEF_ATOM_DB_SCATTXS_TAG == tagname or DEF_ATOM_DB_ABSXS_TAG == tagname:
				list = []
				list.append(val)
				list.append(atr_val)
				info_dic[tagname] = list
		#print 'info_dic', info_dic

		return info_dic

	###########################################################################
	def PutIsotopeNameAndMassList_Name(self, atom_name):
		"""
		atom_nameで指定された原子名の同位体名と質量のリストを返す。
		@param	self									自身のインスタンス
		@param	atom_name		(string)				原子名
		@retval					(list)					同位体名と質量からなるリストの二重リスト
		@retval					(Result)				取得の成否
		"""
		name_mass_list = []
		rslt = Result(RSLT_OK)

		for isotope_dic in self.isoTopeInfo:
			dic_list = []
			if atom_name == isotope_dic[DEF_ATOM_DB_ATOMNAME_TAG]:
				dic_list = isotope_dic[DEF_ATOM_DB_KEY_DICTIONARY]
				for dic in dic_list:
					nm_list = []
					name = dic[DEF_ATOM_DB_NAME_TAG]
					mass = dic[DEF_ATOM_DB_MASS_TAG]
					nm_list.append(name)
					nm_list.append(mass)
					name_mass_list.append(nm_list)
				break
			else:
				continue
		if 0 == len(name_mass_list):
			# リストが存在しないならば、取得失敗
			rslt = Result(RSLT_NG)
			print('NG!')

		return name_mass_list, rslt

	###########################################################################
	def PutIsotopeNameAndMassList_No(self, atom_no):
		"""
		atom_noで指定された原子番号の同位体名と質量のリストを返す。
		@param	self									自身のインスタンス
		@param	atom_no			(string)				原子番号
		@retval					(list)					同位体名と質量からなるリストの二重リスト
		"""
		name_mass_list = []
		rslt = Result(RSLT_OK)

		for isotope_dic in self.isoTopeInfo:
			dic_list = []
			if atom_no == isotope_dic[DEF_ATOM_DB_ID_TAG]:
				dic_list = isotope_dic[DEF_ATOM_DB_KEY_DICTIONARY]
				for dic in dic_list:
					nm_list = []
					name = dic[DEF_ATOM_DB_NAME_TAG]
					mass = dic[DEF_ATOM_DB_MASS_TAG]
					nm_list.append(name)
					nm_list.append(mass)
					name_mass_list.append(nm_list)
				break
			else:
				continue
		if 0 == len(name_mass_list):
			# リストが存在しないならば、取得失敗
			rslt = Result(RSLT_NG)
			print('NG!')

		return name_mass_list, rslt

	###########################################################################
	def PutAtomInfoFromDictonary_No(self, atom_no, tag):
		"""
		atom_noで指定された原子番号に該当する原子の辞書からtagで指定された値を返す。
		@param	self									自身のインスタンス
		@param	atom_no			(string)				原子番号
		@param	tag				(string)				タグ名
		@retval					(string)				原子番号、タグで指定された情報
		@retval					(int)					処理結果
		"""
		dic = {}
		rslt = DEF_ATOM_DB_GET_VALUE
		val = ""
		rv = []

		for atom_dic in self.atomInfo:
			if atom_no == atom_dic[DEF_ATOM_DB_ID_TAG]:
				dic = atom_dic
				#print 'dic = ', dic
				break
			else:
				continue
		if 0 == len(dic):
			# 辞書が存在しないならば、取得失敗
			rslt = DEF_ATOM_DB_ARG_ERROR
			print('NG!')

		if tag in dic:
			val = dic[tag]
			if DEF_ATOM_DB_INCB_TAG == tag and "---" == val[0]:
				# Incbタグ指定時に値が'---'(なし)である場合は取得失敗
				rslt = DEF_ATOM_DB_NOVAL
		else:
			# tagが存在しないならば、取得失敗
			rslt = DEF_ATOM_DB_ARG_ERROR

		if type([]) != type(val):
			# 行列でないなら、戻り値の行列に要素追加
			rv.append(val)
		else:
			# 行列なら、戻り値の行列に連結
			rv.extend(val)
		return rv, rslt

	###########################################################################
	def PutAtomInfoFromDictonary_Name(self, atom_name, tag):
		"""
		atom_nameで指定された原子名に該当する原子の辞書からtagで指定された値を返す。
		@param	atom_no			(string)				原子名
		@param	tag				(string)				タグ名
		@retval					(string)				原子番号、タグで指定された情報
		@retval					(int)					処理結果
		"""
		dic = {}
		rslt = DEF_ATOM_DB_GET_VALUE
		val = ""
		rv = []
		for atom_dic in self.atomInfo:
			if atom_name == atom_dic[DEF_ATOM_DB_ATOMNAME_TAG]:
				dic = atom_dic
				#print 'dic = ', dic
				break
			else:
				continue
		if 0 == len(dic):
			# 辞書が存在しないならば、取得失敗
			rslt = DEF_ATOM_DB_ARG_ERROR
			print('NG!')

		if tag in dic:
			val = dic[tag]

			if DEF_ATOM_DB_INCB_TAG == tag and "---" == val[0]:
				# Incbタグ指定時に値が'---'(なし)である場合は取得失敗
				rslt = DEF_ATOM_DB_NOVAL
		else:
			# tagが存在しないならば、取得失敗
			rslt = DEF_ATOM_DB_ARG_ERROR

		if type([]) != type(val):
			# 行列でないなら、戻り値の行列に要素追加
			rv.append(val)
		else:
			# 行列なら、戻り値の行列に連結
			rv.extend(val)
		return rv, rslt

	###########################################################################
	def PutIsotopeInfoFromDictonary(self, isotope, tag):
		"""
		isotopeで指定された同位体名に該当する原子の辞書からtaで指定された値を返す。
		@param	atom_no			(string)				同位体名
		@param	tag				(string)				タグ名
		@retval					(string)				同位体名、タグで指定された情報
		@retval					(int)					処理結果
		"""
		dic = {}
		rslt = DEF_ATOM_DB_GET_VALUE
		val = ""
		rv = []

		for isotope_dics in self.isoTopeInfo:
			for isotope_dic in isotope_dics[DEF_ATOM_DB_KEY_DICTIONARY]:
				if isotope == isotope_dic[DEF_ATOM_DB_NAME_TAG]:
					dic = isotope_dic
					atom_name = isotope_dics[DEF_ATOM_DB_ATOMNAME_TAG]
					#print 'dic = ', dic
					#print 'atom_name = ', atom_name
					break
				else:
					continue
			if 0 != len(dic):
				# 辞書が見つかったら、ループ終了。
				break

		if 0 == len(dic):
			# 辞書が存在しないならば、取得失敗
			rslt = DEF_ATOM_DB_ARG_ERROR
			print('NG!')
		else:
			#print 'tag = ', tag
			if tag in dic:
				val = dic[tag]
				rslt = DEF_ATOM_DB_GET_VALUE
				if '' == dic[tag]:
					# 値が存在しないならば、原子から取得する。
					rslt = DEF_ATOM_DB_NOVAL
			else:
				# tagが存在しないならば、原子から取得する。
					rslt = DEF_ATOM_DB_NOVAL

			if DEF_ATOM_DB_NOVAL == rslt:
				# 原子から取得する。
				val, rslt = self.PutAtomInfoFromDictonary_Name(atom_name, tag)
				print('@@@rslt = ',rslt)
				if DEF_ATOM_DB_GET_VALUE == rslt:
					# 原子から値取得成功。
					rslt = DEF_ATOM_DB_GET_PARENT_VALUE


		if type([]) != type(val):
			# 行列でないなら、戻り値の行列に要素追加
			rv.append(val)
		else:
			# 行列なら、戻り値の行列に連結
			rv.extend(val)
		return rv, rslt

	###########################################################################
	def GetAtomNameList(self, id_name_list):
		"""
		原子名リストを取得する(GUI周期表用)
		「No id」の順に「AtomName」を配列に取得する。
		配列は2次元にしてNoも取得する。
		引数にはリストを渡さないと、エラーを返すことに注意すること。
		戻り値は成否。
		@param	self									自身のインスタンス
		@param	id_name_list	(list)					取得する番号と原子名のリストからなる二重リスト
		@retval					(Result)				処理結果
		"""
		rslt = Result(RSLT_OK)

		# id_name_listがリスト型でなければNG。
		if type([]) != type(id_name_list):
			rslt = Result(RSLT_NG)
		else:
			# id_name_listのリスト内をクリアする。
			for i, x in enumerate(id_name_list):
				id_name_list.pop(i)

		# self.atomIDNameListが空リストならNG。
		if len(self.atomIDNameList) == 0:
			rslt = Result(RSLT_NG)

		if RSLT_OK == rslt.status:
			for atomlist in self.atomIDNameList:
				id_name_list.append(atomlist)

		return Result(RSLT_OK)

	###########################################################################
	def GetValue(self, key, tag, return_val):
		"""
		原子名or同位体名or原子番号、getする値のタグで指定された値を文字列で取得する
		(アルファベットのみ→原子名、混合→IsoTope、数字→原子番号)
		ただし、
			(1)Isotopeのプロパティがない場合は、原子のタグから値を引いて戻す。(親の値を取得)
			(2)AtomのIncbを取得する場合、値がない場合は、同位体の存在比(conc)とIncbから計算して戻す。(同位体の値を取得)
		(1)(2)もできない場合は、否：値がない

		戻り値は成否。
			成:値を取得(0)、親の値を取得(1)、同位体の値(2)を取得の3種類
			否:引数の間違い(3)、値がない(4)の2種類

		@param	self									自身のインスタンス
		@param	key				(string)				原子名or同位体名or原子番号
		@param	tag				(string)				getする値のタグ
		@param	return_val								引数で指定されたタグの値のリスト(プロパティのUncがあれば、値用の変数を(値、Unc)で戻す)
		@retval					(Result)				処理結果
		"""
		rslt = DEF_ATOM_DB_GET_VALUE
		dic = {}

		# return_valがリストでなければ、引数の間違い。
		if type([]) != type(return_val):
			print('retval_err')
			rslt = DEF_ATOM_DB_ARG_ERROR
		else:
			# return_valのリスト内をクリアする。
			for i, x in enumerate(return_val):
				return_val.pop(i)

		print('key = ',key)
		# keyが文字列でなければ、引数の間違い。
		if type(str()) != type(key):
			print('key_err')
			rslt = DEF_ATOM_DB_ARG_ERROR
		elif 0 == len(key):
			# 空文字列であるなら、引数の間違い。
			print('key_err2')
			rslt = DEF_ATOM_DB_ARG_ERROR

		# tagが文字列でなければ、引数の間違い。
		if type(str()) != type(tag):
			print('tag_err')
			rslt = DEF_ATOM_DB_ARG_ERROR
		elif 0 == len(tag):
			# 空文字列であるなら、引数の間違い。
			print('tag_err2')
			rslt = DEF_ATOM_DB_ARG_ERROR

		val = []
		val_l = []
		taglist = []
		if DEF_ATOM_DB_VIEWCOLORS_TAG == tag:
			# Viewcolorsを指定されたら、赤、緑、青を一度に全て取得すること。
			taglist = [DEF_ATOM_DB_VIEWRED_TAG, DEF_ATOM_DB_VIEWGREEN_TAG, DEF_ATOM_DB_VIEWBLUE_TAG]
		else:
			taglist.append(tag)

		if DEF_ATOM_DB_ARG_ERROR != rslt:
			for i, tag_n in enumerate(taglist):
				#print 'tag_n = ', tag_n
				if True == key.isalpha():
					# アルファベットのみは原子名での指定
					#print'alpha'
					val, rslt = self.PutAtomInfoFromDictonary_Name(key, tag_n)
				elif True == key.isdigit():
					# 数字のみは原子番号での指定
					#print'number'
					val, rslt = self.PutAtomInfoFromDictonary_No(key, tag_n)
				elif True == key.isalnum():
					# 数字と原子番号の混合での指定は同位体
					#print'number+alpha'
					val, rslt = self.PutIsotopeInfoFromDictonary(key, tag_n)
				else:
					# 指定が数字のみ、アルファベットのみでない場合はNG
					rslt = DEF_ATOM_DB_ARG_ERROR
				val_l.extend(val)
			# 引数にリストを設定。
			return_val.extend(val_l)

		return val, rslt

	###########################################################################
	def GetIsotopeNameList(self, atom_info, isotope_name_list):
		"""
		同位体名リストを取得する(GUI周期表用)
		原子名or原子番号で、getする同位体を指定する。
		戻りはMass順にソートする。
		配列は2次元にして、Massも取得する。
		引数にはリストを渡さないと、エラーを返すことに注意すること。
		@param	self									自身のインスタンス
		@param	atom_info		(string)				原子名or原子番号
		@param	isotope_name_list	(list)					取得する同位体名とMassのリストからなる二重リスト
		@retval					(Result)				処理結果
		"""
		isotope_info_list = []
		rslt = Result(RSLT_OK)

		# isotope_name_listがリスト型でなければNG。
		if type([]) != type(isotope_name_list):
			rslt = Result(RSLT_NG)
		else:
			# リスト型なら、リストをクリアする。
			for i, x in enumerate(isotope_name_list):
				isotope_name_list.pop(i)

		# isotope_name_listが空リストでないならNG。
	#	if len(isotope_name_list) !=0:
	#		rslt = Result(RSLT_NG)

		if RSLT_OK == rslt.status:
			if True == atom_info.isalpha():
				# アルファベットのみは原子名での指定
				isotope_info_list, rslt = self.PutIsotopeNameAndMassList_Name(atom_info)
			elif True == atom_info.isdigit():
				# 数字のみは原子番号での指定
				isotope_info_list, rslt = self.PutIsotopeNameAndMassList_No(atom_info)
			else:
				# 指定が数字のみ、アルファベットのみでない場合はNG
				rslt = Result(RSLT_NG)

		if RSLT_OK == rslt.status:
			for list in sorted(isotope_info_list, key = lambda x:x[1], reverse = False):
				isotope_name_list.append(list)
		else:
			print('NG!')

		return rslt

###############################################################################
# 原子名or同位体名or原子番号、getする値のタグで指定された値を文字列で取得する関数
###############################################################################
def GetValue(key, tag, xmlpath = ""):
	"""
	原子名or同位体名or原子番号、getする値のタグで指定された値を文字列で取得する
	(アルファベットのみ→原子名、混合→IsoTope、数字→原子番号)
	ただし、
		(1)Isotopeのプロパティがない場合は、原子のタグから値を引いて戻す。(親の値を取得)
		(2)AtomのIncbを取得する場合、値がない場合は、同位体の存在比(conc)とIncbから計算して戻す。(同位体の値を取得)
	(1)(2)もできない場合は、否：値がない

	戻り値は成否。
		成:値を取得(0)、親の値を取得(1)、同位体の値(2)を取得の3種類
		否:引数の間違い(3)、値がない(4)の2種類
	@param	key				(string)				原子名or同位体名or原子番号
	@param	tag				(string)				getする値のタグ
	@param	xmlpath			(string)				atomDBTaikan.xmlのフルパス
	@retval					(Result)				処理結果
	@retval					(list)					引数で指定されたタグの値のリスト(プロパティのUncがあれば、値用の変数を(値、Unc)で戻す)
	"""

	print('xmlpath:', xmlpath)
	return_val = []
	adt = AtomDBTaikanClass()
	adt.ReadDBFile(xmlpath)

	val, rs = adt.GetValue(key, tag, return_val)

	return rs, return_val

###############################################################################
# 原子名リストを取得する関数
###############################################################################
def GetAtomNameList(xmlpath = ""):
	"""
	原子名リストを取得する(GUI周期表用)
	「No id」の順に「AtomName」を配列に取得する。
	配列は2次元にしてNoも取得する。
	戻り値は成否。
	@param	xmlpath			(string)				atomDBTaikan.xmlのフルパス
	@retval					(Result)				処理結果
	@retval					(list)					番号と原子名のリストからなる二重リストリスト
	"""

	print('xmlpath:', xmlpath)
	list = []
	adt = AtomDBTaikanClass()
	adt.ReadDBFile(xmlpath)

	rs = adt.GetAtomNameList(list)

	return rs, list

###############################################################################
# 原子名リストを取得する関数
###############################################################################
def GetIsotopeNameList(atom_info, xmlpath = ""):
	"""
	同位体名リストを取得する(GUI周期表用)
	原子名or原子番号で、getする同位体を指定する。
	戻りはMass順にソートする。
	配列は2次元にして、Massも取得する。
	@param	atom_info		(string)				原子名or原子番号
	@param	xmlpath			(string)				atomDBTaikan.xmlのフルパス
	@retval					(Result)				処理結果
	@retval					(list)					同位体名とMassのリストからなる二重リスト
	"""

	print('xmlpath:', xmlpath)
	list = ['A']
	adt = AtomDBTaikanClass()
	adt.ReadDBFile(xmlpath)

	rs = adt.GetIsotopeNameList(atom_info, list)

	return rs, list

###############################################################################
# テスト用スクリプト
###############################################################################
if __name__=='__main__':
	#
	# 簡単なクラスのテスト
	#
#import fw.common.ParamUtil as PU
#from fw.common.Result import *
#import os, datetime
#from xml.dom.minidom import parse

	# 原子情報ファイル名
	DEF_ATOM_DB_FILE_NAME = '/home/kfukaya/atomDBTaikan.xml'
	list = []
	print('------  unit Test:Start --------')
	ret, list = GetValue('Al', 'Incb', DEF_ATOM_DB_FILE_NAME)
	print('------  unit Test:End --------')
	print('ret:', ret)
	print('list:', list)

	print('------  unit Test:Start --------')
	ret, list = GetAtomNameList(DEF_ATOM_DB_FILE_NAME)
	print('------  unit Test:End --------')
	print('ret:', ret)
	print('list:', list)

	print('------  unit Test:Start --------')
	#ret, list = GetIsotopeNameList('Al', DEF_ATOM_DB_FILE_NAME)
	ret, list = GetIsotopeNameList('Al')
	print('------  unit Test:End --------')
	print('ret:', ret)
	print('list:', list)




# -*- coding: utf-8 -*-
"""

"""

import utsusemi.vis.VisualModuleQBase as UVM

# 予約語の定義
vdm = None
vmp = None
vm2p = None
vcm = None

ec = None
DAT = None
ECM = None
ECA = None

###################################################


def DetectMap(matrix=DAT):
    """
    Display Detector Map
    @param  matrix [Def:DAT(ElementContainerMatrix)] ElementContainerMatrix Object
    @retval vdm
    """
    return UVM.DetectMap(matrix)

###################################################


def DetectMap2(matrix=DAT):
    """
    Display Detector Map
    @param  matrix [Def:DAT(ElementContainerMatrix)] ElementContainerMatrix Object
    @retval vm2p
    """
    return UVM.DetectMap2(matrix)

###################################################


def DetectMap2ChangeData(vm2p=vm2p, matrix=DAT):
    """
    Display Detector Map
    @param  vm2p [Def:(vis_retval)] Instance of 1D Plotter
    @param  matrix [Def:DAT(ElementContainerMatrix)] ElementContainerMatrix Object
    @retval None
    """
    UVM.DetectMapChangeData(vm2p, matrix)

###################################################


def MPlot(ec=ec):
    """
    Create 1D Plotter and Draw EC
    @param  ec [Def:EC(ElementContainer,ListOfEC)]  ElementContainer
    @param  psd      PSD No.
    @param  pixel    Pixel No.
    @retval vmp    Instance of Plotter
    """
    return UVM.MPlot(ec)

###################################################


def AddToMPlot(vmp=vmp, ec=ec):
    """
    Add EC to 1D Plotter
    @param  vmp [Def:(vis_retval)]     Instance of 1D Plotter
    @param  ec [Def:EC(ElementContainer)]      ElementContainer
    @retval None
    """
    UVM.AddToMPlot(vmp, ec)

###################################################


def EmptyMPlot():
    """
    Create 1D Plotter and Draw EC without data
    @param  None
    @retval vmp Instance of Plotter
    """
    return UVM.MPlot(None)

###################################################


def M2PlotPlus(matrix=ECA):
    """
    Create 2D Plotter and display 2D Map
    @param  matrix [Def:ECM(ElementContainerArray,ElementContainerMatrix)]  ElementContainerArray
    @retval vm2p Instance of 2D Plotter
    """
    return UVM.M2PlotPlus(matrix)

###################################################


def ChangeDataM2Plot(vm2p=vm2p, matrix=ECA):
    """
    Change 2D Map of M2Plot
    @param  vm2p [Def:(vis_retval)]     Instance of 2D Plotter
    @param  matrix [Def:ECA(ElementContainerArray,ElementContainerMatrix)]  ElementContainerArray
    @retval None
    """
    UVM.ChangeDataM2Plot(vm2p, matrix)

###################################################


def EmptyM2PlotPlus():
    """
    Create 2D Plotter and display 2D Map without data
    @param  None
    @retval vm2p  Instance of 2D Plotter
    """
    return UVM.EmptyM2PlotPlus()


# 関数名登録ディクショナリ
_functions = {"DetectMap": DetectMap,
              "DetectMap2": DetectMap2,
              "DetectMap2ChangeData": DetectMap2ChangeData,
              "MPlot": MPlot,
              "AddToMPlot": AddToMPlot,
              "EmptyMPlot": EmptyMPlot,
              "M2PlotPlus": M2PlotPlus,
              "ChangeDataM2Plot": ChangeDataM2Plot,
              "EmptyM2PlotPlus": EmptyM2PlotPlus,
              }

_functionsOrder = [
    "DetectMap",
    "MPlot",
    "AddToMPlot",
    "EmptyMPlot",
    "M2PlotPlus",
    "ChangeDataM2Plot",
    "EmptyM2PlotPlus",
    "DetectMap2",
    "DetectMap2ChangeData",
]



from __future__ import print_function
import numpy

import os
import time


import Manyo as mm
import Manyo.Utsusemi as mu
import threading


class D4Mat(object):
    """
    This is python lapping for commands treating D4Matrix on Manyo-lib
    Required Manyo-lib revision is 2250 or later.
    [inamura 160227] : bugfix of wrong path treatment on Windows Environment
    """

    def __init__(self, Ef=0.0):
        """
        Constructor
        """
        self.SlicedECM = None
        self.D4Mat = mu.UtsusemiD4Matrix()
        if Ef > 0.0:
            self.D4Mat.SetEf(Ef)
        self.CommentHead = "D4Mat >>>"
        self.progress = 0
        self.progress_max = 0
        self.AddedFileList = []

    def new(self, A1range, A2range, A3range, A4range, titles, units, paramfilepath):
        """
        makes new matrix

        @param A1range (list) [A1min, A1max, dA1]
        @param A2range (list) [A2min, A2max, dA2]
        @param A3range (list) [A3min, A3max, dA3]
        @param A4range (list) [A4min, A4max, dA4]
        @param titles  (list of string) [ AX1_title, AX2_title, AX3_title, AX4_title ]
        @param units  (list of string) [ AX1_unit, AX2_unit, AX3_unit, AX4_unit ]
        @param datadir (string)
        @param paramfile (string)
        @return None
        """

        if not (isinstance(A1range, list) and len(A1range) == 3):
            print(self.CommentHead + "Invalid parameter about Q1 range. ")
            raise
        if not (isinstance(A2range, list) and len(A2range) == 3):
            print(self.CommentHead + "Invalid parameter about Q2 range. ")
            raise
        if not (isinstance(A3range, list) and len(A3range) == 3):
            print(self.CommentHead + "Invalid parameter about Q3 range. ")
            raise
        if not (isinstance(A4range, list) and len(A4range) == 3):
            print(self.CommentHead + "Invalid parameter about hw range. ")
            raise
        if not (isinstance(titles, list) and len(titles) == 4):
            print(self.CommentHead + "Invalid parameter about titles. ")
            raise
        if not (isinstance(units, list) and len(units) == 4):
            print(self.CommentHead + "Invalid parameter about units. ")
            raise
        if not isinstance(paramfilepath, str):
            print(self.CommentHead + "Invalid parameter about param file. ")
            raise

        a1v = mm.ListToDoubleVector(A1range)
        a2v = mm.ListToDoubleVector(A2range)
        a3v = mm.ListToDoubleVector(A3range)
        a4v = mm.ListToDoubleVector(A4range)
        tlv = mm.MakeStringVector()
        unv = mm.MakeStringVector()
        for title, unit in zip(titles, units):
            tlv.append(str(title))
            unv.append(str(unit))
        # [inamura 160227]-->  # for Windows Environment
        # datadir = os.path.dirname( paramfilepath ) + "/"
        # paramfile = paramfilepath.replace( datadir, "" )
        datadir = os.path.dirname(paramfilepath)
        paramfile = os.path.basename(paramfilepath)
        # <--[inamura 160227]
        self.D4Mat.AllocateNewMat(
            a1v, a2v, a3v, a4v, tlv, unv, datadir, paramfile)

        if not self.D4Mat.isStatusOK():
            print(self.CommnetHead + "Fail to allocate New matrix.. ")
            raise

    def open(self, paramfilepath):
        """
        Opens existed Matrix data

        @param paramfilepath (string) full path to parameter xml file of D4Matrix
        @retval -1  Error happened
        @retval  0  No trouble
        """
        if not os.path.exists(paramfilepath):
            print(self.CommentHead + "Cannot find file " + paramfilepath)
            return -1

        # [inamura 160227]-->  # for Windows Environment
        # datadir = os.path.dirname( paramfilepath ) + "/"
        # filename = paramfilepath.replace( datadir, "" )
        datadir = os.path.dirname(paramfilepath)
        filename = os.path.basename(paramfilepath)
        # <--[inamura 160227]
        print("{} Data dir = {}".format(self.CommentHead, datadir))
        print("{} param_file = {}".format(self.CommentHead, filename))
        self.D4Mat.OpenMat(datadir, filename)

        if not self.D4Mat.isStatusOK():
            print(self.CommentHead + "Fail to open Matrix.. ")
            return -1

        return 0

    def close(self):
        """
        Closes opened Matrix data

        @param None
        @retval None
        """

        self.D4Mat.CloseMat()

    def rebin(self, paramfile1, paramfile2):
        """
        Rebins a matrix to the other matrix

        @param paramfile1    full path to param xml file of original matrix
        @param paramfile2    full path to param xml file of matrix to be rebined
        @retval  -1   Error happened
        @retval   0   No trouble
        """
        if not os.path.exists(paramfile1):
            print(self.CommentHead + "Cannot find file " + paramfile1)
            return -1
        if not os.path.exists(paramfile2):
            print(self.CommentHead + "Cannot find file " + paramfile2)
            return -1

        # [inamura 160227]-->  # for Windows Environment
        # datadir1 = os.path.dirname( paramfile1 ) + "/"
        # filename1 = paramfile1.replace( datadir1, "" )
        # datadir2 = os.path.dirname( paramfile2 ) + "/"
        # filename2 = paramfile2.replace( datadir2, "" )
        datadir1 = os.path.dirname(paramfile1)
        filename1 = os.path.basename(paramfile1)
        datadir2 = os.path.dirname(paramfile2)
        filename2 = os.path.basename(paramfile2)
        # <--[inamura 160227]

        print(self.CommentHead + "Rebin from:")
        print(self.CommentHead + "      Data dir = {}".format(datadir1))
        print(self.CommentHead + "      param_file = {}".format(filename1))
        print(self.CommentHead + "Rebin to:")
        print(self.CommentHead + "      Data dir = {}".format(datadir2))
        print(self.CommentHead + "      param_file = {}".format(filename2))

        self.D4Mat.CloseMat()
        self.D4Mat.AddMatToMat(str(datadir1), str(filename1), str(datadir2), str(filename2))

        if not self.D4Mat.isStatusOK():
            print(self.CommentHead + "Fail to rebin Matrix.. ")
            raise

        return 0

    def isOpened(self):
        """
        Returns True if any matrix data has been opened.
        @param None
        @retval True   matrix data has been opened.
        @retval False  not opened.
        """
        return self.D4Mat.isMatOpened()

    def putOpenedPath(self):
        """
        Returns path to parameter xml file of opened D4Matrix

        @param None
        @retval tupple of string ( directory, xml_file_name )
        """
        ret = self.D4Mat.PutOpenedDataPath()
        if ret.size() < 2:
            return None

        return (ret[0], ret[1])

    def setTitles(self, titles, isSaved=False):
        """
        Sets other titles

        @param titles ( list of string ) [AX1 title,AX2 title,AX3 title,AX4 title]
        @param isSaved  (bool) If True, rewrite parameter xml file
        @return None
        """
        if isinstance(titles, list) and len(titles) == 4:
            pass
        else:
            print(self.CommentHead + " First Argument must be a list of 4 strings. ")
            raise
        if not isinstance(isSaved, bool):
            print(self.CommentHead + " Second Argument must be bool. ")
            raise

        tv = mm.MakeStringVector()
        for title in titles:
            tv.append(str(title))
        self.D4Mat.SetAxTitles(tv, isSaved)

        # for i,title in enumerate(titles):
        #    self.D4Mat.SetAxTitle( i, str(title) )

    def setUnits(self, units, isSaved=False):
        """
        Sets other units

        @param units ( list of string ) [AX1 unit,AX2 unit,AX3 unit,AX4 unit]
        @param isSaved  (bool) If True, rewrite parameter xml file
        @return None
        """
        if isinstance(units, list) and len(units) == 4:
            pass
        else:
            print(self.CommentHead + " First Argument must be a list of 4 strings. ")
            raise
        if not isinstance(isSaved, bool):
            print(self.CommentHead + " Second Argument must be bool. ")
            raise
        tv = mm.MakeStringVector()
        for unit in units:
            tv.append(str(unit))
        self.D4Mat.SetAxUnits(tv, isSaved)

    def addFromDirectry(self, filedir):
        """
        Adds data from files by given directory

        @param filedir (string) path to directory stored text file
        @return List of file path to be loaded
        """
        if not os.path.exists(filedir):
            print(self.CommnetHead + "Cannot find such directory, {}".format(filedir))
            raise

        filelist = os.listdir(filedir)
        filelist.sort()

        num_files = len(filelist)

        self.AddedFileList = []
        self.progress = 0
        self.progress_max = num_files
        print("self.progress_max=", self.progress_max)
        for filename in filelist:
            filepath = os.path.join(filedir, filename)
            msg = self.CommentHead + "filepath = " + filepath
            msg += " :%d/%d" % ((self.progress + 1), num_files)
            print(msg)
            if filepath is not None:
                time.sleep(0.2)
                self.AddedFileList.append(self.addFromFile(filepath))
            self.progress += 1

        self.progress += 1

        print(self.CommentHead + "--------------------------------------------------")
        print(self.CommentHead + " Finished : Load vbin files")
        print(self.CommentHead + " in %s" % (filedir))
        print(self.CommentHead + "--------------------------------------------------")

        return self.AddedFileList

    def addFromFile(self, filename):
        """
        Adds data from a file

        @param filename (string) path to data file
        @return file path to be loaded
        """
        st = time.time()
        try:
            if filename[-5:] == ".vbin":
                self.D4Mat.AddToMatFromBin(str(filename))
            elif filename[-4:] == ".txt":
                self.D4Mat.AddToMatFromText(str(filename))
            else:
                print(self.CommentHead + " File name is invalid. ")
                raise

            if not self.D4Mat.isStatusOK():
                print(self.CommnetHead + "Fail to add data from file.. ")
                raise

        except:
            print(self.CommentHead + "Cannot read file : {}".format(filename))
            filename = None
        print(self.CommentHead + "time of cost = {}".format(time.time() - st))
        return filename

    def subtractText(self, filename):
        """
        Subtracts data of text file from Matrix

        @param filename (string) path to data of text file
        @retval -1  Error happened
        @retval  0  No trouble
        """

        try:
            self.D4Mat.SubtractFromMatByText(str(filename))
            if not self.D4Mat.isStatusOK():
                print(self.CommnetHead + "Fail to subtract text file data from D4Mat.. ")
                return -1
        except:
            print(self.CommentHead + "Fail to open text file..")
            return -1

        return 0

    def sliceMat(self, A1range, A2range, A3range, A4range, type_axes, foldings):
        """
        Slices Matrix

        @param A1range    (list of float) range of AX1
        @param A2range    (list of float) range of AX2
        @param A3range    (list of float) range of AX3
        @param A4range    (list of float) range of AX4
        @param type_axes  (list of string) type of axes 'X','Y' or 'T'
        @param foldings   (list of int )
        @return None
        """

        if not (isinstance(A1range, list) and (len(A1range) == 2)):
            print(self.CommnetHead + "Invalid parameter about Q1 range. ")
            raise
        if not (isinstance(A2range, list) and (len(A2range) == 2)):
            print(self.CommnetHead + "Invalid parameter about Q2 range. ")
            raise
        if not (isinstance(A3range, list) and (len(A3range) == 2)):
            print(self.CommnetHead + "Invalid parameter about Q3 range. ")
            raise
        if not (isinstance(A4range, list) and (len(A4range) == 2)):
            print(self.CommnetHead + "Invalid parameter about hw range. ")
            raise
        if not (isinstance(type_axes, list) and (len(type_axes) == 4)):
            print(self.CommnetHead + "Invalid type_axes. ")
            raise

        a1v = mm.ListToDoubleVector(A1range)
        a2v = mm.ListToDoubleVector(A2range)
        a3v = mm.ListToDoubleVector(A3range)
        a4v = mm.ListToDoubleVector(A4range)

        cnt_Ax = {'X': 0, 'Y': 0, 'T': 0}
        def_axes = mm.MakeStringVector()
        isValid = True
        for Axtype in type_axes:
            if Axtype == 'X':
                cnt_Ax['X'] += 1
            elif Axtype == 'Y':
                cnt_Ax['Y'] += 1
            elif Axtype == 'T':
                cnt_Ax['T'] += 1
            else:
                break
            def_axes.append(Axtype)

        dim = 0
        if (cnt_Ax['X'] == 1) and (cnt_Ax['Y'] == 1) and (cnt_Ax['T'] == 2):
            dim = 2
        elif (cnt_Ax['X'] == 1) and (cnt_Ax['T'] == 3):
            dim = 1
        else:
            # if (cnt_Ax['X']!=1) or (cnt_Ax['Y']!=1) or (cnt_Ax['T']!=2):
            print(self.CommentHead + "Invalid argument type_axes.")
            isValid = False

        fld = mm.ListToDoubleVector(foldings)  # [inamura 160616]
        if isValid:
            if dim == 2:
                self.D4Mat.SliceMat(a1v, a2v, a3v, a4v, def_axes, fld)
            else:
                self.D4Mat.SliceMat1d(a1v, a2v, a3v, a4v, def_axes, fld)
            if not self.D4Mat.isStatusOK():
                print(self.CommnetHead + "Fail to slice data.. ")
                raise
            self.SlicedECM = self.D4Mat.PutSlicedECA()
        else:
            raise

    # [inamura 170227]-->
    def sliceMat3d(self, A1range, A2range, A3range, A4range, type_axes, foldings, name_axes):
        """
        Slices Matrix 3D

        @param A1range    (list of float) range of AX1
        @param A2range    (list of float) range of AX2
        @param A3range    (list of float) range of AX3
        @param A4range    (list of float) range of AX4
        @param type_axes  (list of string) type of axes 'X','Y','Z' or 'T'
        @param foldings   (list of int )
        @return None
        """

        if not (isinstance(A1range, list) and (len(A1range) == 2)):
            print(self.CommnetHead + "Invalid parameter about Q1 range. ")
            raise
        if not (isinstance(A2range, list) and (len(A2range) == 2)):
            print(self.CommnetHead + "Invalid parameter about Q2 range. ")
            raise
        if not (isinstance(A3range, list) and (len(A3range) == 2)):
            print(self.CommnetHead + "Invalid parameter about Q3 range. ")
            raise
        if not (isinstance(A4range, list) and (len(A4range) == 2)):
            print(self.CommnetHead + "Invalid parameter about hw range. ")
            raise
        if not (isinstance(type_axes, list) and (len(type_axes) == 4)):
            print(self.CommnetHead + "Invalid type_axes. ")
            raise

        a1v = mm.ListToDoubleVector(A1range)
        a2v = mm.ListToDoubleVector(A2range)
        a3v = mm.ListToDoubleVector(A3range)
        a4v = mm.ListToDoubleVector(A4range)

        cnt_Ax = {'X': 0, 'Y': 0, 'Z': 0, 'T': 0}
        def_axes = mm.MakeStringVector()
        isValid = True
        for Axtype in type_axes:
            if Axtype == 'X':
                cnt_Ax['X'] += 1
            elif Axtype == 'Y':
                cnt_Ax['Y'] += 1
            elif Axtype == 'Z':
                cnt_Ax['Z'] += 1
            elif Axtype == 'T':
                cnt_Ax['T'] += 1
            else:
                break
            def_axes.append(Axtype)

        if (cnt_Ax['X'] != 1) or (cnt_Ax['Y'] != 1) or (cnt_Ax['Y'] != 1) or (cnt_Ax['T'] != 1):
            print(self.CommentHead + "Invalid argument type_axes.")
            isValid = False

        fld = mm.ListToDoubleVector(foldings)
        keys = mm.MakeStringVector(3)
        for i in range(3):
            keys[i] = str(name_axes[i])

        if isValid:
            try:
                del self.SlicedECM
            except:
                pass
            self.SlicedECM = mm.ElementContainerMatrix()
            self.D4Mat.SliceMat3D(self.SlicedECM, a1v, a2v,
                                  a3v, a4v, def_axes, fld, keys)
            if not self.D4Mat.isStatusOK():
                print(self.CommnetHead + "Fail to slice data.. ")
                raise
        else:
            raise
        # <--[inamura 170227]

    def loadVisContParams(self, filename=""):
        """
        Pick up information of Axis from VisCont parameter file

        @param filename   path to XML parameter file produced by VisualCont2
        @return  tupple of list ( list_of_ranges_for_axes, list_of_titles_for_axes )
        """
        if not os.path.exists(filename):
            ss = self.CommentHead + (" Cannot find XML file({})".format(filename))
            raise UserWarning("FileIOError::{}".format(ss))

        vcp = mu.UtsusemiSqeCalcXtalParams()
        if vcp.LoadFile(filename):
            pass
        else:
            msg = self.CommentHead + \
                ("Failed load XtalParam file, may be this is not XtalParam ({})".format(
                    filename))
            raise UserWarning(msg)

        axrange_list = []
        title_list = []
        unit_list = []

        ax_type = vcp.PutAxType()
        ax_labels = vcp.PutViewAxesLabels()
        ax_units = vcp.PutViewAxesUnits()
        for i in range(4):
            ax_range = vcp.PutAxRange(i)
            r_axis = []
            for j in range(ax_range.size()):
                r_axis.append(ax_range[j])
            axrange_list.append(r_axis)
            title_list.append(ax_labels[i])
            unit_list.append(ax_units[i])

        return (axrange_list, title_list, unit_list)

    def makeNewMatFromVisContParams(self, VisContParamFile="", D4MatParamFile=""):
        """
        Makes new D4Matrix data file using parameters fo VisContParam

        @param VisContParamFile   path to XML parameter file produced by VisualCont2
        @param D4MatParamFile     path to XML parameter file for D4Matrix
        @return None
        """
        (range_list, title_list) = self.loadVisContParams(VisContParamFile)

        self.new(range_list[0], range_list[1], range_list[2],
                 range_list[3], title_list, D4MatParamFile)



import threading
import time
import wx
import wx.lib.newevent
import sys
SHOW_PROGRESS        = True
(ExecStartEvent , EVT_EXEC_START ) = wx.lib.newevent.NewEvent()
(ExecEndEvent   , EVT_EXEC_END   ) = wx.lib.newevent.NewEvent()
(ProgressEvent  , EVT_PROGRESS   ) = wx.lib.newevent.NewEvent()
class ExecCommandCommon:
    def __init__(self,func,event=None,args=(),objs=[],onfunc=None,offfunc=None):
        self.func   = func
        self.event  = event
        self.objs   = objs
        self.onfunc = onfunc
        self.offfunc= offfunc

        if self.event!=None:
            self.obj=event.GetEventObject()
            self.obj.Bind(EVT_EXEC_START,self.execCommandStart)
            self.obj.Bind(EVT_EXEC_END,  self.execCommandEnd)
            if SHOW_PROGRESS:
                self.progress=None
                self.obj.Bind(EVT_PROGRESS,  self.progressCommand)
        else:
            self.obj=None

        for obj in self.objs:
            obj.Bind(EVT_EXEC_START,self.execCommandStart)
            obj.Bind(EVT_EXEC_END  ,self.execCommandEnd)

        if isinstance(args, tuple):
            self.args=args
            pass
        elif isinstance(args, tuple):
            self.args=args
            pass
        else:
            self.args=(args,)

        self.threadProcess   = threading.Thread(target=self.func, args=self.args)
        self.watchDogProcess = threading.Thread(target=self.watchDog)
        self.execCommand()
#-----------------------------------------------------------------------------------
    def execCommand(self):
        if self.obj!=None:
            evt = ExecStartEvent()
            evt.SetEventObject(self.obj)
            wx.PostEvent(self.obj, evt)

        if len(self.objs)>0:
            for obj in self.objs:
                evt = ExecStartEvent()
                evt.SetEventObject(obj)
                wx.PostEvent(obj, evt)

        if self.onfunc!=None:
            self.onfunc()

        self.threadProcess.start()
        self.watchDogProcess.start()
#-----------------------------------------------------------------------------------
    def watchDog(self):

        count =0
        while 1:
            if self.threadProcess.isAlive():
                if self.obj!=None and SHOW_PROGRESS:
                    evt = ProgressEvent(count=count)
                    evt.SetEventObject(self.obj)
                    wx.PostEvent(self.obj, evt)
                time.sleep(1)
                count +=1
            else:
                break

        if self.obj!=None and SHOW_PROGRESS:
            evt = ProgressEvent(count=-1)
            evt.SetEventObject(self.obj)
            wx.PostEvent(self.obj, evt)
            time.sleep(1)

        if self.obj!=None:
            evt = ExecEndEvent()
            evt.SetEventObject(self.obj)
            wx.PostEvent(self.obj, evt)

        if len(self.objs)>0:
            for obj in self.objs:
                evt = ExecEndEvent()
                evt.SetEventObject(obj)
                wx.PostEvent(obj, evt)

        if self.offfunc!=None:
            self.offfunc()
#-----------------------------------------------------------------------------------
    def progressCommand(self,event):

        if event.count==0:
            self.progress = wx.ProgressDialog("Progress",
                                    "            Process is running now     ",
                                    maximum = 3600,
                                    #style = wx.PD_CAN_ABORT
                                     #| wx.PD_CAN_SKIP
                                    style = wx.PD_ELAPSED_TIME
                                     #| wx.PD_APP_MODAL
                                     #| wx.PD_ESTIMATED_TIME
                                     #| wx.PD_REMAINING_TIME
                                     )

        elif event.count==-1 and self.progress!=None:
            self.progress.Destroy()
            self.pregress=None

        elif self.progress!=None:
            val=event.count%3600
            self.progress.Update(val,"")

        else:
            pass
#-----------------------------------------------------------------------------------
    def execCommandStart(self,event):
        obj=event.GetEventObject()
        obj.Disable()
#-----------------------------------------------------------------------------------
    def execCommandEnd(self,event):
        obj=event.GetEventObject()
        obj.Enable()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'DetectMapQ.ui'
#
# Created: Mon Jan 20 19:07:46 2020
#      by: pyside-uic 0.2.15 running on PySide 1.2.2
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_mapframe(object):
    def setupUi(self, mapframe):
        mapframe.setObjectName("mapframe")
        mapframe.resize(985, 778)
        self.centralwidget = QtGui.QWidget(mapframe)
        self.centralwidget.setObjectName("centralwidget")
        self.groupBox = QtGui.QGroupBox(self.centralwidget)
        self.groupBox.setGeometry(QtCore.QRect(10, 10, 301, 51))
        self.groupBox.setObjectName("groupBox")
        self.tof1 = QtGui.QLineEdit(self.groupBox)
        self.tof1.setGeometry(QtCore.QRect(10, 20, 80, 25))
        self.tof1.setObjectName("tof1")
        self.tof2 = QtGui.QLineEdit(self.groupBox)
        self.tof2.setGeometry(QtCore.QRect(110, 20, 80, 25))
        self.tof2.setObjectName("tof2")
        self.label = QtGui.QLabel(self.groupBox)
        self.label.setGeometry(QtCore.QRect(100, 20, 16, 25))
        self.label.setObjectName("label")
        self.btRedraw = QtGui.QPushButton(self.groupBox)
        self.btRedraw.setGeometry(QtCore.QRect(200, 20, 80, 30))
        self.btRedraw.setObjectName("btRedraw")
        self.groupBox_2 = QtGui.QGroupBox(self.centralwidget)
        self.groupBox_2.setGeometry(QtCore.QRect(10, 80, 301, 51))
        self.groupBox_2.setObjectName("groupBox_2")
        self.psd0 = QtGui.QLineEdit(self.groupBox_2)
        self.psd0.setGeometry(QtCore.QRect(80, 20, 80, 25))
        self.psd0.setObjectName("psd0")
        self.bt2dPlot = QtGui.QPushButton(self.groupBox_2)
        self.bt2dPlot.setGeometry(QtCore.QRect(170, 20, 100, 30))
        self.bt2dPlot.setObjectName("bt2dPlot")
        self.label_2 = QtGui.QLabel(self.groupBox_2)
        self.label_2.setGeometry(QtCore.QRect(20, 20, 66, 17))
        self.label_2.setObjectName("label_2")
        self.groupBox_3 = QtGui.QGroupBox(self.centralwidget)
        self.groupBox_3.setGeometry(QtCore.QRect(320, 10, 211, 121))
        self.groupBox_3.setObjectName("groupBox_3")
        self.label_3 = QtGui.QLabel(self.groupBox_3)
        self.label_3.setGeometry(QtCore.QRect(70, 30, 66, 17))
        self.label_3.setObjectName("label_3")
        self.label_4 = QtGui.QLabel(self.groupBox_3)
        self.label_4.setGeometry(QtCore.QRect(160, 30, 41, 17))
        self.label_4.setObjectName("label_4")
        self.label_5 = QtGui.QLabel(self.groupBox_3)
        self.label_5.setGeometry(QtCore.QRect(20, 50, 41, 25))
        self.label_5.setObjectName("label_5")
        self.label_6 = QtGui.QLabel(self.groupBox_3)
        self.label_6.setGeometry(QtCore.QRect(20, 90, 41, 25))
        self.label_6.setObjectName("label_6")
        self.PSD1 = QtGui.QLineEdit(self.groupBox_3)
        self.PSD1.setGeometry(QtCore.QRect(70, 50, 50, 25))
        self.PSD1.setObjectName("PSD1")
        self.PSD2 = QtGui.QLineEdit(self.groupBox_3)
        self.PSD2.setGeometry(QtCore.QRect(70, 90, 50, 25))
        self.PSD2.setObjectName("PSD2")
        self.pixel1 = QtGui.QLineEdit(self.groupBox_3)
        self.pixel1.setGeometry(QtCore.QRect(150, 50, 50, 25))
        self.pixel1.setObjectName("pixel1")
        self.pixel2 = QtGui.QLineEdit(self.groupBox_3)
        self.pixel2.setGeometry(QtCore.QRect(150, 90, 50, 25))
        self.pixel2.setObjectName("pixel2")
        self.label_7 = QtGui.QLabel(self.groupBox_3)
        self.label_7.setGeometry(QtCore.QRect(130, 50, 16, 25))
        self.label_7.setObjectName("label_7")
        self.label_8 = QtGui.QLabel(self.groupBox_3)
        self.label_8.setGeometry(QtCore.QRect(130, 90, 16, 25))
        self.label_8.setObjectName("label_8")
        self.cbXaxisOfPlot = QtGui.QComboBox(self.centralwidget)
        self.cbXaxisOfPlot.setGeometry(QtCore.QRect(560, 10, 100, 27))
        self.cbXaxisOfPlot.setObjectName("cbXaxisOfPlot")
        self.cbXaxisOfPlot.addItem("")
        self.cbXaxisOfPlot.addItem("")
        self.cbXaxisOfPlot.addItem("")
        self.btAppend = QtGui.QPushButton(self.centralwidget)
        self.btAppend.setGeometry(QtCore.QRect(560, 40, 150, 27))
        self.btAppend.setObjectName("btAppend")
        self.btSumm = QtGui.QPushButton(self.centralwidget)
        self.btSumm.setGeometry(QtCore.QRect(560, 70, 150, 27))
        self.btSumm.setObjectName("btSumm")
        self.btNewPlot = QtGui.QPushButton(self.centralwidget)
        self.btNewPlot.setGeometry(QtCore.QRect(560, 100, 150, 27))
        self.btNewPlot.setObjectName("btNewPlot")
        self.label_9 = QtGui.QLabel(self.centralwidget)
        self.label_9.setGeometry(QtCore.QRect(720, 10, 66, 17))
        self.label_9.setObjectName("label_9")
        self.ckconv = QtGui.QCheckBox(self.centralwidget)
        self.ckconv.setGeometry(QtCore.QRect(810, 10, 151, 22))
        self.ckconv.setObjectName("ckconv")
        self.plotList = QtGui.QListWidget(self.centralwidget)
        self.plotList.setGeometry(QtCore.QRect(720, 30, 200, 115))
        self.plotList.setObjectName("plotList")
        self.slider = QtGui.QSlider(self.centralwidget)
        self.slider.setGeometry(QtCore.QRect(0, 230, 30, 350))
        self.slider.setProperty("value", 99)
        self.slider.setOrientation(QtCore.Qt.Vertical)
        self.slider.setInvertedAppearance(False)
        self.slider.setObjectName("slider")
        self.mapPanel = QtGui.QWidget(self.centralwidget)
        self.mapPanel.setGeometry(QtCore.QRect(30, 160, 950, 500))
        self.mapPanel.setObjectName("mapPanel")
        self.groupBox_4 = QtGui.QGroupBox(self.centralwidget)
        self.groupBox_4.setGeometry(QtCore.QRect(30, 670, 401, 51))
        self.groupBox_4.setObjectName("groupBox_4")
        self.ymin = QtGui.QLineEdit(self.groupBox_4)
        self.ymin.setGeometry(QtCore.QRect(20, 20, 100, 25))
        self.ymin.setObjectName("ymin")
        self.ymax = QtGui.QLineEdit(self.groupBox_4)
        self.ymax.setGeometry(QtCore.QRect(130, 20, 100, 25))
        self.ymax.setObjectName("ymax")
        self.label_10 = QtGui.QLabel(self.groupBox_4)
        self.label_10.setGeometry(QtCore.QRect(120, 20, 16, 25))
        self.label_10.setObjectName("label_10")
        self.btIntSet = QtGui.QPushButton(self.groupBox_4)
        self.btIntSet.setGeometry(QtCore.QRect(240, 20, 80, 30))
        self.btIntSet.setObjectName("btIntSet")
        self.btIntClear = QtGui.QPushButton(self.groupBox_4)
        self.btIntClear.setGeometry(QtCore.QRect(340, 20, 50, 30))
        self.btIntClear.setObjectName("btIntClear")
        self.infotxt = QtGui.QLabel(self.centralwidget)
        self.infotxt.setGeometry(QtCore.QRect(450, 670, 500, 55))
        self.infotxt.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.infotxt.setObjectName("infotxt")
        self.line = QtGui.QFrame(self.centralwidget)
        self.line.setGeometry(QtCore.QRect(0, 140, 981, 16))
        self.line.setFrameShape(QtGui.QFrame.HLine)
        self.line.setFrameShadow(QtGui.QFrame.Sunken)
        self.line.setObjectName("line")
        mapframe.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(mapframe)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 985, 25))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtGui.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuDisplay = QtGui.QMenu(self.menubar)
        self.menuDisplay.setObjectName("menuDisplay")
        mapframe.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(mapframe)
        self.statusbar.setObjectName("statusbar")
        mapframe.setStatusBar(self.statusbar)
        self.actionOpen = QtGui.QAction(mapframe)
        self.actionOpen.setObjectName("actionOpen")
        self.actionSave = QtGui.QAction(mapframe)
        self.actionSave.setObjectName("actionSave")
        self.actionSave_as = QtGui.QAction(mapframe)
        self.actionSave_as.setObjectName("actionSave_as")
        self.actionPrint = QtGui.QAction(mapframe)
        self.actionPrint.setObjectName("actionPrint")
        self.actionExit = QtGui.QAction(mapframe)
        self.actionExit.setObjectName("actionExit")
        self.actionNew_Plotter = QtGui.QAction(mapframe)
        self.actionNew_Plotter.setObjectName("actionNew_Plotter")
        self.actionLog_Scale = QtGui.QAction(mapframe)
        self.actionLog_Scale.setCheckable(True)
        self.actionLog_Scale.setObjectName("actionLog_Scale")
        self.menuFile.addAction(self.actionOpen)
        self.menuFile.addAction(self.actionSave)
        self.menuFile.addAction(self.actionSave_as)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionPrint)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionExit)
        self.menuDisplay.addAction(self.actionNew_Plotter)
        self.menuDisplay.addAction(self.actionLog_Scale)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuDisplay.menuAction())

        self.retranslateUi(mapframe)
        QtCore.QMetaObject.connectSlotsByName(mapframe)

    def retranslateUi(self, mapframe):
        mapframe.setWindowTitle(QtGui.QApplication.translate("mapframe", "Detect Map", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate("mapframe", "X-axis Range", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("mapframe", "-", None, QtGui.QApplication.UnicodeUTF8))
        self.btRedraw.setText(QtGui.QApplication.translate("mapframe", "Redraw", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_2.setTitle(QtGui.QApplication.translate("mapframe", "GroupBox", None, QtGui.QApplication.UnicodeUTF8))
        self.bt2dPlot.setText(QtGui.QApplication.translate("mapframe", "2D Plot", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("mapframe", "DetNo. :", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_3.setTitle(QtGui.QApplication.translate("mapframe", "DetId Range", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("mapframe", "Detector", None, QtGui.QApplication.UnicodeUTF8))
        self.label_4.setText(QtGui.QApplication.translate("mapframe", "Pixel", None, QtGui.QApplication.UnicodeUTF8))
        self.label_5.setText(QtGui.QApplication.translate("mapframe", "From", None, QtGui.QApplication.UnicodeUTF8))
        self.label_6.setText(QtGui.QApplication.translate("mapframe", "To", None, QtGui.QApplication.UnicodeUTF8))
        self.label_7.setText(QtGui.QApplication.translate("mapframe", "-", None, QtGui.QApplication.UnicodeUTF8))
        self.label_8.setText(QtGui.QApplication.translate("mapframe", "-", None, QtGui.QApplication.UnicodeUTF8))
        self.cbXaxisOfPlot.setItemText(0, QtGui.QApplication.translate("mapframe", "TOF", None, QtGui.QApplication.UnicodeUTF8))
        self.cbXaxisOfPlot.setItemText(1, QtGui.QApplication.translate("mapframe", "Detector", None, QtGui.QApplication.UnicodeUTF8))
        self.cbXaxisOfPlot.setItemText(2, QtGui.QApplication.translate("mapframe", "Pixel", None, QtGui.QApplication.UnicodeUTF8))
        self.btAppend.setText(QtGui.QApplication.translate("mapframe", "Append >>", None, QtGui.QApplication.UnicodeUTF8))
        self.btSumm.setText(QtGui.QApplication.translate("mapframe", "Append Sum >>", None, QtGui.QApplication.UnicodeUTF8))
        self.btNewPlot.setText(QtGui.QApplication.translate("mapframe", "New Plotter", None, QtGui.QApplication.UnicodeUTF8))
        self.label_9.setText(QtGui.QApplication.translate("mapframe", "Plotters", None, QtGui.QApplication.UnicodeUTF8))
        self.ckconv.setText(QtGui.QApplication.translate("mapframe", "Convert toEnergy", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox_4.setTitle(QtGui.QApplication.translate("mapframe", "Intensity Range", None, QtGui.QApplication.UnicodeUTF8))
        self.label_10.setText(QtGui.QApplication.translate("mapframe", "-", None, QtGui.QApplication.UnicodeUTF8))
        self.btIntSet.setText(QtGui.QApplication.translate("mapframe", "Apply", None, QtGui.QApplication.UnicodeUTF8))
        self.btIntClear.setText(QtGui.QApplication.translate("mapframe", "Clear", None, QtGui.QApplication.UnicodeUTF8))
        self.infotxt.setText(QtGui.QApplication.translate("mapframe", "Det-ID", None, QtGui.QApplication.UnicodeUTF8))
        self.menuFile.setTitle(QtGui.QApplication.translate("mapframe", "File", None, QtGui.QApplication.UnicodeUTF8))
        self.menuDisplay.setTitle(QtGui.QApplication.translate("mapframe", "Display", None, QtGui.QApplication.UnicodeUTF8))
        self.actionOpen.setText(QtGui.QApplication.translate("mapframe", "Open ...", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSave.setText(QtGui.QApplication.translate("mapframe", "Save", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSave_as.setText(QtGui.QApplication.translate("mapframe", "Save as ...", None, QtGui.QApplication.UnicodeUTF8))
        self.actionPrint.setText(QtGui.QApplication.translate("mapframe", "Print ...", None, QtGui.QApplication.UnicodeUTF8))
        self.actionExit.setText(QtGui.QApplication.translate("mapframe", "Exit", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNew_Plotter.setText(QtGui.QApplication.translate("mapframe", "New Plotter ...", None, QtGui.QApplication.UnicodeUTF8))
        self.actionLog_Scale.setText(QtGui.QApplication.translate("mapframe", "Log Scale", None, QtGui.QApplication.UnicodeUTF8))




from __future__ import print_function
import Manyo as mm
import Cmm
import scipy
import scipy.optimize
import numpy
#-------------------------------------------------------------------------------------------------
def GetHistogram():
    return 0
#-------------------------------------------------------------------------------------------------
def MovingAverage(ECM,sample):
    ecm=mm.ElementContainerMatrix()
    ecaSize = ECM.PutTableSize()
    for i in range(ecaSize):
        eca=MovingAverageECA(ECM.PutPointer(i),sample)
        ecm.Add(eca)
    return ecm
#-------------------------------------------------------------------------------------------------
def CalcTOFOffset(ECM, energyMin, energyMax,
                       energyLimitMin=-1, energyLimitMax=-1,
                       peakLimitMin=-1, peakLimitMax=-1,
                       outputFilename="tofOffset.txt", failedFilename="failedData.txt"):
    offsetArray,failedData = CalcOriginOfEnergy(ECM,energyMin,energyMax,
                                   energyLimitMin,energyLimitMax,peakLimitMin,peakLimitMax)

    WriteFitFailedData(failedFilename, failedData)
    WriteTOFOffset(outputFilename, offsetArray)
#-------------------------------------------------------------------------------------------------
def MovingAverageECA(ECA,sample):
    if sample >1:

        eca    = mm.ElementContainerArray(ECA.PutHeaderPointer())
        ecSize = ECA.PutTableSize()
        ec     = ECA.PutPointer(0)
        xkey,ykey,ekey    = [ec.PutXKey(),ec.PutYKey(),ec.PutEKey()]
        xunit,yunit,eunit = [ec.PutUnit(key) for key in [xkey,ykey,ekey]]


        totalCount=ec.PutHeaderPointer().PutDouble("TotalCounts")
        for k in range(1,sample-1):
            ecTmp = ECA.PutPointer(k)
            ec = ec + ecTmp
            totalCount += ecTmp.PutHeaderPointer().PutDouble("TotalCounts")


        for j in range(ecSize-sample):
            ecj         = ECA.PutPointer(j)
            ech         = ecj.PutHeaderPointer()
            ecjCount    = ech.PutDouble("TotalCounts")

            ecTmp       = ECA.PutPointer(sample-1+j)
            ec          = ec + ecTmp
            totalCount += ecTmp.PutHeaderPointer().PutDouble("TotalCounts")

            ech.OverWrite("TotalCounts",totalCount)
            ec.InputHeader(ech)
            ec.SetUnit(xkey,xunit)
            ec.SetUnit(ykey,yunit)
            ec.SetUnit(ekey,eunit)
            eca.Add(ec)

            ec          = ec - ecj
            totalCount -= ecjCount

        ec          = ec + ecj
        ec.SetUnit(xkey,xunit)
        ec.SetUnit(ykey,yunit)
        ec.SetUnit(ekey,eunit)
        totalCount += ecjCount

        for l in range(ecSize-sample,ecSize):
            ech= ECA.PutPointer(l).PutHeaderPointer()
            ech.OverWrite("TotalCounts",totalCount)
            #ec.InputHeader(ech)
            eca.Add(ec)
    else:
        eca=ECA
    return eca

#-------------------------------------------------------------------------------------------------
def PutValArray(ec,energyMin,energyMax):
    ecXn = numpy.array(ec.PutXList())
    ecX  = (ecXn[1:]+ecXn[:-1])*0.5
    ecY  = numpy.array(ec.PutYList())

    minReginArray = ecX>energyMin
    maxReginArray = ecX<energyMax
    regionArray   = numpy.logical_and(minReginArray,maxReginArray)

    ecX           = ecX[regionArray]
    ecY           = ecY[regionArray]
    return ecX,ecY
#-------------------------------------------------------------------------------------------------
def CalcOriginOfEnergy(ECM,energyMin,energyMax,
                       energyLimitMin=-1,energyLimitMax=-1,peakLimitMin=-1,peakLimitMax=-1):
    ecaSize = ECM.PutTableSize()
    failedData=[]
    offsetPsds=[]
    xarray,yarray = PutValArray(ECM.Put(0).Put(0),energyMin,energyMax)
    for i in range(ecaSize):
        ECA=ECM.PutPointer(i)
        ecSize=ECA.PutTableSize()
        offsetPixels=[]
        for j in range(ecSize):
            EC=ECA.PutPointer(j)

            mask = EC.PutHeaderPointer().PutInt4("MASKED")
            if mask==1:
                offset   =  0.0
            else:
                xarray,yarray = PutValArray(EC,energyMin,energyMax)
                guess = EstimateFittingInitial(xarray,yarray)
                try:
                    param = ExecGaussFit(xarray,yarray,guess)
                    flag,msg = CheckLimitCondition(param,energyLimitMin,energyLimitMax,
                                                         peakLimitMin,  peakLimitMax)
                    if flag:
                        offset = CalcTOFOffsetVal(EC,param[1])
                    else:
                        print("   Out of Range [",msg,"] (psd=%d,pixel=%d)"%(i,j))
                        failedData.append([i,j,xarray,yarray])
                        offset   = 0.0

                except Exception as e:
                    if "Number of calls to function has reached maxfev" in str(e):
                        print("   Number of calls to function has reached maxfev (psd=%d,pixel=%d)"%(i,j))
                    else:
                        print("   "+str(e)+"(psd=%d,pixel=%d)"%(i,j))
                    failedData.append([i,j,xarray,yarray])
                    offset   =  0.0
            offsetPixels.append(offset)
        offsetPsds.append(offsetPixels)
    return offsetPsds,failedData
#-------------------------------------------------------------------------------------------------
def WriteTOFOffset(filename,offset_array):
    f=open(filename,"w")
    for array in offset_array:
        text = ""
        for val in array:
            text += "%.4f "%val
        text = text.strip()+"\n"
        f.write(text)
    f.close()
#-------------------------------------------------------------------------------------------------
def WriteFitFailedData(filename,failedData):

    f=open(filename,"w")
    for psd,pixel,dataX,dataY in failedData:
        text="# psd:%d, pixel:%d"%(psd,pixel)+"\n"
        f.write(text)

        for x,y in zip(dataX,dataY):
            text ="%f %f\n"%(x,y)
            f.write(text)

        text ="\n"
        f.write(text)
    f.close()
#-------------------------------------------------------------------------------------------------
def CheckLimitCondition(param,energyLimitMin=-1,energyLimitMax=-1,peakLimitMin=-1,peakLimitMax=-1):
    if energyLimitMin!=energyLimitMax:
        if param[1]<energyLimitMin:
            return False,"Lower Energy"
        if param[1]>energyLimitMax:
            return False,"Higher Energy"
    if peakLimitMin!=peakLimitMax:
        if param[0]<peakLimitMin:
            return False,"Lower Peak"
        if param[0]>peakLimitMax:
            return False,"Higher Peak"

    return True,""
#-------------------------------------------------------------------------------------------------
def GaussFunction(x,height,center,sigma,constant):

    return height*numpy.exp( -0.5*((x-center)/sigma)**2 ) + constant
#-------------------------------------------------------------------------------------------------
def ExecGaussFit(xarray,yarray,guess):
    #popt,pconv = scipy.optimize.curve_fit(GaussFunction,xarray,yarray,guess,maxfev=1000)
    popt,pconv = scipy.optimize.curve_fit(GaussFunction,xarray,yarray,guess)

    return popt
#-------------------------------------------------------------------------------------------------
def ExecGaussFitEC(ec,energyMin,energyMax,height,center,fwhm,back):
    ecX,ecY=PutValArray(ec,energyMin,energyMax)
    if height<0 and center<0 and fwhm<0 and back<0:
        height,center,fwhm,back = EstimateFittingInitial(ecX,ecY)

    popt = ExecGaussFit(ecX,ecY,[height,center,fwhm,back])
    newX = numpy.arange(ecX[0],ecX[-1],(ecX[-1]-ecX[0])/1000.0)
    newY = GaussFunction(newX,popt[0],popt[1],popt[2],popt[3])
    ecOut= mm.ElementContainer()

    newX -= (newX[1] - newX[0])*0.5
    newX = numpy.hstack([newX,numpy.array([newX[-1]+(newX[1] - newX[0])])])
    ecOut.Add("x",newX.tolist())
    ecOut.Add("y",newY.tolist())
    ecOut.Add("e",numpy.zeros(len(newY)).tolist())
    ecOut.SetKeys("x","y","e")
    return ecOut
#-------------------------------------------------------------------------------------------------
def CalcTOFOffsetVal(EC,dHW):
    ech=EC.PutHeaderPointer()

    ef = ech.PutDouble("Ef")
    l1 = ech.PutDouble("L1")
    #l2 = ech.PutDouble("L2")
    #l3 = ech.PutDouble("L3")
    TOF_offset = 2286.2869 * l1 * ( 1.0/numpy.sqrt(ef) - 1.0/numpy.sqrt(ef+dHW) )

    return TOF_offset
#-------------------------------------------------------------------------------------------------
# Estimate Gaussian Function
#-------------------------------------------------------------------------------------------------
def EstimateFittingInitial(x,y):
    sumVal     = GetIntegral(x,y)
    maxY,maxX  = GetMaximum(x,y)
    binwidth   = GetBinWidth(x,y)
    fwhm       = GetFWHM(sumVal,maxY,binwidth)
    background = 0.0
    return maxY,maxX,fwhm,background
#-------------------------------------------------------------------------------------------------
def GetBinWidth(x,y):
    #binwidth=x[1]-x[0]
    binwidth = numpy.float(x.max()-x.min())/x.size
    return binwidth
#-------------------------------------------------------------------------------------------------
def GetIntegral(x,y):
    return float(y.sum())
#-------------------------------------------------------------------------------------------------
def GetMaximum(x,y):
    #z=zip(y,x)
    #return max(z)
    sumY=y.sum()
    if sumY==0:
        return 0.0,0.0
    else:
        return numpy.float(y.max()),numpy.float((x*y).sum())/y.sum()
#-------------------------------------------------------------------------------------------------
def GetFWHM(area,height,binwidth):
    if height==0:
        return 1
    else:
        return area/height/numpy.sqrt(2*numpy.pi)*numpy.sqrt(2*numpy.log(2))*(binwidth)
#-------------------------------------------------------------------------------------------------
